import React, { useEffect, useState, useMemo } from 'react'
import {
    Table,
    Button,
    Row,
    Col,
    Input,
    Modal,
    Form,
    Select,
    message,
    Drawer,
    Descriptions,
} from 'antd'
import { ColumnProps } from 'antd/lib/table'
import {
    useAccessControllList,
    useAddAccessControl,
} from '../../services/accessControl'
import { useProjectList } from '../../services/project'
import { BASIC_ADD_FORMS } from './AddForm'
const { Search } = Input
const DeviceState = {
    0: '离线',
    1: '在线',
}
const ApplyDustStatus = {
    0: '待确认',
    1: '已确认',
    2: '非本项目',
}
const colums: ColumnProps<any>[] = [
    {
        title: '项目名',
        dataIndex: 'project_name',
        key: 'project_name',
    },
    {
        title: '设备名',
        dataIndex: 'name',
        key: 'name',
    },
    {
        title: '设备唯一编码',
        dataIndex: 'sn',
        key: 'sn',
    },
    {
        title: '状态',
        dataIndex: 'state',
        key: 'state',
        render: (dataKey) => DeviceState[dataKey],
    },
    {
        title: '申请时间',
        dataIndex: 'created_time',
        key: 'created_time',
    },
    {
        title: '确认状态',
        dataIndex: 'status',
        key: 'status',
        render: (dataKey) => ApplyDustStatus[dataKey],
    },
    {
        title: '审核时间',
        dataIndex: 'approve_time',
        key: 'approve_time',
    },
]

const TYPE_OPTIONS = {
    1: '进场设备',
    2: '出场设备',
    3: '进出场设备',
    4: '采集设备',
}
const TYPE_CATEGORYS = {
    1: '虹膜设备',
    2: '红外人脸设备',
    3: '可见光人脸设备',
    4: '可见光+红外人脸设备',
    5: '可见光+虹膜设备',
}
const addForms = BASIC_ADD_FORMS.concat([
    {
        name: 'kind',
        label: '设备类型',
        placeholder: '请选择设备类型',
        required: true,
        top: 31,
        component: (props) => (
            <Select {...props}>
                {Object.keys(TYPE_OPTIONS).map((element) => {
                    return (
                        <Select.Option value={+element} key={element}>
                            {TYPE_OPTIONS[element]}
                        </Select.Option>
                    )
                })}
            </Select>
        ),
    },
    {
        name: 'sub_kind',
        label: '设备类别',
        placeholder: '请选择设备类型',
        top: 32,
        required: true,
        component: (props) => (
            <Select {...props}>
                {Object.keys(TYPE_CATEGORYS).map((element) => {
                    return (
                        <Select.Option value={+element} key={element}>
                            {TYPE_CATEGORYS[element]}
                        </Select.Option>
                    )
                })}
            </Select>
        ),
    },
])
const detailsConfig = [
    {
        name: '项目信息',
        key: 'project',
        children: [
            {
                name: '项目名',
                key: 'project_name',
            },
            {
                name: '项目安检备案号',
                key: 'safety_record_no',
            },
        ],
    },
    {
        name: '设备信息',
        key: 'device',
        children: [
            {
                name: '生产厂商',
                key: 'manufacturer',
            },
            {
                name: '设备型号',
                key: 'model',
            },
            {
                name: '生产批次',
                key: 'batch',
            },
            {
                name: '设备名',
                key: 'name',
            },
            {
                name: '联系人',
                key: 'person',
            },
            {
                name: '联系电话',
                key: 'phone',
            },
            {
                name: '设备唯一编码',
                key: 'sn',
            },
            {
                name: '对接单位',
                key: 'unit',
            },
            {
                name: '设备类型',
                key: 'kind',
            },
            {
                name: '设备类别',
                key: 'sub_kind',
            },
        ],
    },
    {
        name: '审批信息',
        key: 'review',
        children: [
            {
                name: '审批时间',
                key: 'approve_time',
            },
        ],
    },
]

/**
 * @description 门禁设备管理列表
 * @param props
 * @returns
 */
const accessControl: React.FC = () => {
    const [form] = Form.useForm()
    const [detailModalState, changeDetailModalState] = useState<{
        visible: boolean
        values?: object
    }>({
        visible: false,
    })
    const [addModalState, changeAddModalState] = useState({ visible: false })
    const [searchState, changeSearchState] = useState({ filter: '', page: 1 })
    const { state: postState, request: requestPost } = useAddAccessControl()
    const { state: listState, request: requestList } = useAccessControllList()
    const [projectListState, requestPorjects] = useProjectList()

    const tableColums = colums.concat({
        title: '操作',
        render: (record) => (
            <Button
                onClick={() => {
                    changeDetailModalState({ visible: true, values: record })
                }}
            >
                详情
            </Button>
        ),
    })

    useEffect(() => {
        if (addModalState.visible) {
            requestPorjects()
        }
    }, [addModalState.visible])
    useEffect(() => {
        requestList(searchState)
    }, [searchState])
    const handleSubmit = () => {
        form.validateFields().then((data) => {
            requestPost(data)
                .then(() => {
                    message.success('添加设备成功')
                    changeAddModalState({ visible: false })
                    requestList(searchState)
                })
                .catch((error) => {
                    message.error(error.message)
                })
        })
    }
    const postForms = useMemo(() => {
        return addForms
            .concat([
                {
                    name: 'project_id',
                    label: '项目',
                    required: true,
                    top: 33,
                    placeholder: '请选择项目',
                    component: (props) => (
                        <Select
                            showSearch
                            filterOption={(inputValue, option) => {
                                return option?.title.indexOf(inputValue) !== -1
                            }}
                            loading={projectListState.loading}
                            {...props}
                        >
                            {projectListState.dataSouce.map((project) => {
                                return (
                                    <Select.Option
                                        title={project.name}
                                        key={project.id}
                                        value={project.id}
                                    >
                                        {project.name}
                                    </Select.Option>
                                )
                            })}
                        </Select>
                    ),
                },
            ])
            .sort((a, b) => b.top - a.top)
    }, [projectListState.dataSouce])
    return (
        <>
            <Row justify="space-between" gutter={[0, 30]}>
                <Col>
                    <Search
                        placeholder="请输入搜索内容"
                        onSearch={(filter) => {
                            changeSearchState({ page: 1, filter })
                        }}
                    ></Search>
                </Col>
                <Col>
                    <Button
                        type="primary"
                        onClick={() => changeAddModalState({ visible: true })}
                    >
                        新增
                    </Button>
                </Col>
            </Row>
            <Modal
                maskClosable={false}
                title="新增"
                visible={addModalState.visible}
                width={600}
                onOk={handleSubmit}
                onCancel={() => changeAddModalState({ visible: false })}
                okButtonProps={{ loading: postState.loading }}
            >
                <Form form={form} labelCol={{ span: 5 }}>
                    {postForms.map((itemForm) => {
                        return (
                            <Form.Item
                                rules={[
                                    {
                                        required: itemForm.required,
                                        message: itemForm.placeholder,
                                    },
                                ]}
                                {...itemForm}
                                key={itemForm.name}
                            >
                                <itemForm.component
                                    placeholder={itemForm.placeholder}
                                ></itemForm.component>
                            </Form.Item>
                        )
                    })}
                </Form>
            </Modal>
            <Table
                loading={listState.loading}
                columns={tableColums}
                dataSource={listState.dataSource}
                pagination={{
                    current: listState.page,
                    hideOnSinglePage: true,
                    showSizeChanger: false,
                    onChange: (page) => {
                        changeSearchState((preState) => ({ ...preState, page }))
                    },
                }}
            ></Table>
            {detailModalState.values ? (
                <Drawer
                    onClose={() => changeDetailModalState({ visible: false })}
                    width={800}
                    title="详情"
                    visible={detailModalState.visible}
                >
                    {detailsConfig.map((configs) => {
                        return (
                            <Descriptions title={configs.name}>
                                {configs.children.map((item) => {
                                    return (
                                        <Descriptions.Item label={item?.name}>
                                            {
                                                (detailModalState.values as object)[
                                                    item?.key
                                                ]
                                            }
                                        </Descriptions.Item>
                                    )
                                })}
                            </Descriptions>
                        )
                    })}
                </Drawer>
            ) : null}
        </>
    )
}
export default accessControl
